/* gtamilocr.h. This file is part of GNU Tamil OCR
 *
 * GNU Tamil OCR :  <http://gtamilocr.sourceforge.net>
 * Free, Cross-Platform, Optical Character Recognition for Tamil
 *
 * Copyright (C) 2004-2007  Visweswara R, <visweswara@users.sourceforge.net>.
 * All Rights Reserved. DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR
 * THIS FILE HEADER.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License version 3 for more details(a copy is included
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * Contributions: GUI enhancement, splash screen ,noise removal functions
 * by Janagaraj V,janagaraj@users.sourceforge.net
 */

#include <wx/wxprec.h>

#include <wx/longlong.h>
#include "imagebox.h"
#include "train.h"
#include "logwin.h"
#include <wx/utils.h>
#include <wx/file.h>
#include <wx/wfstream.h>
#include <wx/tipdlg.h>
#include <wx/filedlg.h>
#include <wx/log.h>
#include <wx/icon.h>
#include "global.h"
#include <wx/splash.h>
#include <wx/fontdlg.h>
#include <wx/strconv.h>
#include <wx/progdlg.h>
#include <wx/mstream.h>
#include <wx/richtext/richtextctrl.h>
#include "gtamilocr_menutool.h"
#include "gtamilocr_utils.h"
#include "aboutdialog.h"
#include "tamilsplash.xpm"
#include "sample.xpm"

//!Namespace for the Image Processing Library (ipl) library.
/*! Namespace for the Image Processing Library (ipl) system. This prevents nameclash with classes from other libraries.
    Using namespace ipl avoids the use of  ipl:: before every ipl98 class
*/
using namespace ipl;

wxString filename;
/*!
bitt is bitamp wxBitmap object used for painting the image on the image frame(imagebox),
this is used global
*/
wxBitmap bitt=wxNullBitmap;

class MyFeatureDialog: public wxDialog
{
public:
    //! Default Constructor
    MyFeatureDialog( wxWindow *parent, wxWindowID id, const wxString &title,
        const wxPoint& pos = wxDefaultPosition,
        const wxSize& size = wxDefaultSize,
        long style = wxDEFAULT_DIALOG_STYLE );

    wxCheckBox* GetCbnormalized()  { return (wxCheckBox*) FindWindow( ID_CBNORMALIZED ); }
    wxCheckBox* GetCbskeleton()  { return (wxCheckBox*) FindWindow( ID_CBSKELETON ); }
    wxCheckBox* GetCbzerinike()  { return (wxCheckBox*) FindWindow( ID_CBZERINIKE ); }
    wxCheckBox* GetCbzone64()  { return (wxCheckBox*) FindWindow( ID_CBZONE64 ); }
    wxCheckBox* GetCbzone16()  { return (wxCheckBox*) FindWindow( ID_CBZONE16 ); }
    wxCheckBox* GetCbmomentin()  { return (wxCheckBox*) FindWindow( ID_CBMOMENTIN ); }
    wxCheckBox* GetCbcompact()  { return (wxCheckBox*) FindWindow( ID_CBCOMPACT ); }

private:
    DECLARE_EVENT_TABLE()
};

//! Main Frame of GNU Tamil OCR
/*!
MyTamilFrame is derived from wxFrame . This class implements is the main frame of the application
*/

class MyTamilFrame: public wxFrame
{
public:
    //! Default constructor.
    MyTamilFrame( wxWindow *parent, wxWindowID id, const wxString &title,
        const wxPoint& pos = wxDefaultPosition,
        const wxSize& size = wxDefaultSize,
        long style = wxMINIMIZE_BOX | wxMAXIMIZE_BOX | wxRESIZE_BORDER | wxSYSTEM_MENU | wxCAPTION| wxMAXIMIZE );
        MyImageBox *imagebox;
		wxTextCtrl *tamiltextctrl;
		MyTrain *train;

		struct db
		{
		double moin[16][4];
		double hiwi;
		wxChar ccode[3];
		}testdb,fdb;

      double u00,u01,u10,u11,u20,u02,u21,u12,u30,u03;
      double i1,i2,i3,i4,m1,m2,m3,m4,yy1,yy2,yy3,yy4;
	  double L00,L01,L10,L11,L12,L21,L20,L02,L30,L03;

	int zoomlevel; //!< zoomlevel is a global variable to store the current zoom level of the image.
/*!< \n if \c \b zoomlevel = 0 The image is diplayed as it is, in <b>original size</b>
     \n if \c \b zoomlevel = 1 The image is scaled <b> one half </b> of the original size
	 \n if \c \b zoomlevel = 2 The image is scaled <b> one fourth </b> of the original size
	 \n if \c \b zoomlevel = 3 The image is scaled <b> one eighth <b> of the original size.

*/

private:
    //!method to create the menubar for MyTamilFrame.
    void CreateMyMenuBar();
	//!method to create the toolbar for MyTamilFrame.
    void CreateMyToolBar();


	void recognize();
    void OnUsinghelp( wxCommandEvent &event );
    void OnSegment( wxCommandEvent &event );
    void OnFit( wxCommandEvent &event );
    void OnZoomout( wxCommandEvent &event );
    void OnZoomin( wxCommandEvent &event );
    void OnSkew( wxCommandEvent &event );
    void OnRotate270( wxCommandEvent &event );
    void OnRotate180( wxCommandEvent &event );
    void OnRotate90( wxCommandEvent &event );
    void OnRevimage( wxCommandEvent &event );
    void OnClear( wxCommandEvent &event );
    void OnOptions( wxCommandEvent &event );
    void OnBatch( wxCommandEvent &event );
    void OnRead( wxCommandEvent &event );
	void OnFont( wxCommandEvent &event );
    void OnSavetextas( wxCommandEvent &event );
    void OnSaveimageas( wxCommandEvent &event );
    void OnSave( wxCommandEvent &event );
    void OnAbout( wxCommandEvent &event );
    void OnFeature( wxCommandEvent &event );
    void OnQuit( wxCommandEvent &event );
    void OnCloseWindow( wxCloseEvent &event );
    void OnSize( wxSizeEvent &event );
    void OnPaint( wxPaintEvent &event );
	void freshredraw();
    void createtemp();
    void OnOpen( wxCommandEvent &event );


    DECLARE_EVENT_TABLE()
};
//! Main wxApp for GNU Tamil OCR
//*Class is derived from wxApp, This is the main wxApp class of GNU Tamil OCR.*/
class MyTamilApp: public wxApp
{
public:
    //!Default constructor.
	MyTamilApp();
    virtual bool OnInit();
    virtual int OnExit();
};
