/* gtamilocr_utils.h. This file is part of GNU Tamil OCR
 *
 * GNU Tamil OCR :  <http://gtamilocr.sourceforge.net>
 * Free, Cross-Platform, Optical Character Recognition for Tamil
 *
 * Copyright (C) 2004-2007  Visweswara R, <visweswara@users.sourceforge.net>.
 * All Rights Reserved. DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR
 * THIS FILE HEADER.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License version 3 for more details(a copy is included
 * in the LICENSE file that accompanied this code).
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#include "gtamilocr_utils.h"
double fact(int n)
{
	int i;
	double s=1.0;
    if(n==0)
    return 1;
    for(i=1;i<=n;i++)
    s*=i;
    return s;
}

double legcoeff(int n,int k)
{
	double cnk=0.0;
	if((n-k)%2==1)
		return 0.0;
	cnk=(pow((-1),(n-k)/2)*fact(n+k))/( pow(2,n)*fact((n-k)/2)*fact((n+k)/2)*fact(k)) ;
	return cnk;
}

double L(int p,int q,CFeatures ftl)
{
	int r,s;
	double u[4][4];
	u[0][0]=ftl.m_Moments.m00;
	u[1][0]=ftl.m_Moments.m10;
	u[0][1]=ftl.m_Moments.m01;
	u[1][1]=ftl.m_Moments.m11;
	u[2][0]=ftl.m_Moments.m20;
	u[0][2]=ftl.m_Moments.m02;
	u[2][1]=ftl.m_Moments.m21;
	u[1][2]=ftl.m_Moments.m12;
	u[3][0]=ftl.m_Moments.m30;
	u[0][3]=ftl.m_Moments.m03;

	double temp1=0,temp2=0;
	temp1=((2*p+1)*(2*q+1))/4;
	for(r=0;r<=p;r++)
	{
			for(s=0;s<=q;s++)
			{
				temp2=temp2+u[r][s]*legcoeff(p,r)*legcoeff(q,s);
			}
	}

return temp2*temp1;

}

double* findhistogram(CImage Img)
{

		    unsigned int row;
            double* histvect=new double[256];
			for(int i=0;i<256;i++)
			 {
				 histvect[i]=0;
			 }

			for(row=0;row<Img.GetHeight();row++)
			{
			CImage::ConstColumnIterator8bp it=Img.ConstColumn8bp(row);

			while(it!=it.End())
			   {
				int pixvalue=*it++;
				histvect[pixvalue]++;
			   }
			}
			double totalpixels=Img.GetWidth()*Img.GetHeight();
			for(int i=0;i<256;i++)
			{
			 histvect[i]=(histvect[i] / totalpixels)*1000;
			}
			return histvect;

}


/*
This otsu algorithm is adopted from gamera toolkit
which is adopted from

  Permission to use, copy, modify and distribute this software and its
  documentation for any purpose and without fee is hereby granted,
  provided that this copyright notice appear in all copies and that
  both that copyright notice and this permission notice appear in supporting
  documentation and that the name of B-lab, Department of Informatics or
  University of Oslo not be used in advertising or publicity pertaining
  to distribution of the software without specific, written prior permission.

  B-LAB DISCLAIMS ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
  IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL B-LAB
  BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
  WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
  OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN
  CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.


*/

int findotsu_threshold(CImage Img)
 {
  int thresh;
  double criterion;
  double expr_1;
  int i, k;
  double omega_k;
  double sigma_b_k;
  double sigma_T;
  double mu_T;
  double mu_k;
  int k_low, k_high;

  double* p = findhistogram(Img);

  mu_T = 0.0;
  for (i=0; i<256; i++)
    mu_T += i*p[i];

  sigma_T = 0.0;
  for (i=0; i<256; i++)
    sigma_T += (i-mu_T)*(i-mu_T)*p[i];


  for (k_low = 0; (p[k_low] == 0) && (k_low < 255); k_low++);
  for (k_high = 255; (p[k_high] == 0) && (k_high > 0); k_high--);

  criterion = 0.0;
  thresh = 127;

  omega_k = 0.0;
  mu_k = 0.0;
  for (k = k_low; k <= k_high ; k++)
    {
      omega_k += p[k];
      mu_k += k*p[k];

      expr_1 = (mu_T*omega_k - mu_k);
      sigma_b_k = expr_1 * expr_1 / (omega_k*(1-omega_k));
      if (criterion < sigma_b_k/sigma_T)
	{
	  criterion = sigma_b_k/sigma_T;
	  thresh = k;
	}
    }
  delete p;
  return (256-thresh);
}


double findskew(CImage Img)
{


   CImage ImgDest;
   CSegmentate seg;
   CPixelGroups pg;
   CFeatures ft;

	int i;
	double thex;


   unsigned int count;

   Img.SetBorderSize(10);
   Img.SetBorderColor(0);

   ImgDest.SetBorderSize(10);
   ImgDest.SetBorderColor(0);

   {
        CMorphology morph;
		ImgDest.CopyConvert(1,Img);

        /* invert image */
		/**change the polarity of the image*/
        ImgDest.Invert();



        for(count=0; count<30; count++)
        {
		   morph.DilateFast(ImgDest,0,3,0);

        }
		ImgDest.Save("01d.bmp");
        for(count=0; count<2; count++)
        {
		   morph.ErodeFast(ImgDest,1,0,1);

        }
		ImgDest.Save("01e.bmp");
        seg.DeriveBlobs(ImgDest,pg,HIGHCOLOR,128,EIGHTCONNECTED,1);
        double sknum=0,skdenom=0;

		double theta;

        for(i=0;i<= (pg.GetTotalGroups()-1);i++)
        {
        ft.DeriveMomentsBinary(pg[i],2);
        ft.DeriveCentralMoments();
        if( (ft.m_CentrMom.m20!=ft.m_CentrMom.m02) && (ft.m_CentrMom.m11!=0) )
         {
          thex=(atan((2*ft.m_CentrMom.m11)/((ft.m_CentrMom.m20)-(ft.m_CentrMom.m02)))/2);
          sknum=sknum+(thex*ft.m_CentrMom.m00);
          skdenom=skdenom+ft.m_CentrMom.m00;
         }

        }

		theta=(sknum/skdenom); //*180/(3.141592654);
		return theta;


   }
}

/** funtion returns  optimal threshold value of the image*/
   int findthresholdsimple(CImage Img)
    {
	int newThreshold,threshold;
	int upperValueTotal,lowerValueTotal,m1,m2;
    // initial estimate for the threshold
	newThreshold = 128;

	do{
        threshold = newThreshold;
     	upperValueTotal = lowerValueTotal = m1 = m2 = 0;
	for (int row=0;row<Img.GetWidth();row++)
	    {
		for (int col=0;col<Img.GetHeight();col++)
		    {

			   if(Img.GetPixelFast(row,col) > threshold)
			    {
				m1 = m1 + Img.GetPixelFast(row,col);
			        upperValueTotal++;
			    }
			else
			    {
				m2 = m2 + Img.GetPixelFast(row,col);
			        lowerValueTotal++;

			    }
		    }

	    }
	 // mean value for each region
		m1 = m1 / upperValueTotal;
        m2 = m2 / lowerValueTotal;
		newThreshold =(m1 + m2)/2;

	}
	while(newThreshold != threshold);

	    return newThreshold;
   }
